#!/bin/bash

echo
echo
echo This is an Installation Validation Program for DeepGreen.
echo It will install a sample DeepGreen Database on one machine.
echo
echo After this script runs successfully, you should have a running DeepGreen Database named
echo "dgtestdb", that you can connect to using the command:
echo "  psql dgtestdb"
echo
echo You can also load sample data into "dgtestdb" using the command:
echo "  ./load-data.sh"
echo
echo "If this script fails to execute, look in the file \"IVP.log\" for the reason why."
echo
echo
sleep 2

rm -f IVP.log

. path.sh

#Set up the port numbers for our sample cluster:
DGHost_PORT=5432

DGHost_LOG=IVP.dgtestdb.log

#Include the functions that support this script.
. ivp_functions.bash

CWD=`pwd`

#Set up the data directories for our sample cluster:
DGHost_DIR=$CWD/dgtestdb

if [ -d $DGHost_DIR ]; then
  echo -n "Removing previous installation, ok? (type 'yes' to proceed) "
  read ans
  if [ $ans = "yes" ]; then
    stopInstance $DGHost_DIR IVP.log
    removeDirectory $DGHost_DIR IVP.log
  else
    echo "Terminating script at your request..."
    exit 1
  fi
fi


echo -n "Creating the dgtestdb data directory..."
  createDatadirectory $DGHost_DIR IVP.log
echo " done."

echo -n "Starting the instance located in dgtestdb ..."
  startInstance $DGHost_PORT $DGHost_DIR $DGHost_LOG off IVP.log
echo " done."

echo -n "Creating the database \"dgtestdb\" ..."
  runSqlCmd $DGHost_PORT "create database dgtestdb" template1 IVP.log
echo " done."

echo -n "Creating table \"bigtable1\" in \"dgtestdb\"..."
  runSqlCmd $DGHost_PORT "create schema IVP;create table IVP.bigtable1 ( a timestamp, b varchar, c varchar, d integer, e integer, f integer, g integer, h numeric, i varchar, j varchar, k varchar, l integer, m integer, n varchar, o varchar )" dgtestdb IVP.log
echo " done."

echo '---------------------------------------------------------------'
echo '---------------------------------------------------------------'
echo '---------------------------------------------------------------'
echo
echo "Congratulations: A DeepGreen database has been installed successfully!"
echo
echo "You can now connect to the database using the command:"
echo "  psql -p 5432 dgtestdb"
echo
echo You can now load sample data into "dgtestdb" using the command:
echo "  ./load-data.sh"
echo
