#!/bin/bash

exitFail()
{
  echo " failed, exiting"
  exit 1
}

doInstance() {
  local PORT_NUM=$1
  local DATA_DIR=$2
  local LOGFILE=$3
  local CDBMODE=$4
  local OUTPUT=$5
  local MODE=$6

  export PGPORT=$PORT_NUM
  pg_ctl -w -D $DATA_DIR -l $LOGFILE -o "-i -p $PGPORT " $MODE 2>&1 1>> $OUTPUT
  if [ $? -eq 1 ]; then exitFail; fi
}

stopInstance() {
  local DATA_DIR=$1
  local OUTPUT=$2

  pg_ctl -D $DATA_DIR stop 2>&1 1>> $OUTPUT
}

startInstance() {
  local PORT_NUM=$1
  local DATA_DIR=$2
  local LOGFILE=$3
  local CDBMODE=$4
  local OUTPUT=$5

  doInstance $PORT_NUM $DATA_DIR $LOGFILE $CDBMODE $OUTPUT start
}

restartInstance() {
  local PORT_NUM=$1
  local DATA_DIR=$2
  local LOGFILE=$3
  local CDBMODE=$4
  local OUTPUT=$5

  doInstance $PORT_NUM $DATA_DIR $LOGFILE $CDBMODE $OUTPUT restart
}

createDatadirectory() {
  local DIR=$1
  local OUTPUT=$2

  initdb $DIR 2>&1 1>> $OUTPUT
  if [ $? -eq 1 ]; then exitFail; fi
}

removeDirectory() {
  local DIR=$1
  local OUTPUT=$2
  echo 
  if [ -x $DIR ]; then
    echo -n "Removing $DIR, ok? (type 'yes' to proceed) "
    read ans
    if [ $ans = "yes" ]; then
      rm -rf $DIR 2>&1 1>> $OUTPUT
    fi
  fi
  if [ $? -eq 1 ]; then exitFail; fi
}

runSqlFile() {
  local PORT=$1
  local FILE=$2
  local DBMSNAME=$3
  local OUTPUT=$4

  psql -p $PORT $DBMSNAME -f $FILE 2>&1 1>> $OUTPUT
  if [ $? -eq 1 ]; then exitFail; fi
}

runSqlCmd() {
  local PORT=$1
  local CMD=$2
  local DBMSNAME=$3
  local OUTPUT=$4

  psql -p $PORT $DBMSNAME -c "$CMD" 2>&1 1>> $OUTPUT
  if [ $? -eq 1 ]; then exitFail; fi
}
